@@ -15,10 +15,11 @@ use Data::Dumper; $Data::Dumper::Indent = 1;
 
 use Net::Z3950::FOLIO::Session;
 use Net::Z3950::FOLIO::OPACXMLRecord qw(makeOPACXMLRecord);
-use Net::Z3950::FOLIO::RPN;;
+use Net::Z3950::FOLIO::RPN;
+use Net::Z3950::FOLIO::SurrogateDiagnostic;
 
 
-our $VERSION = 'v4.0.1';
+our $VERSION = 'v4.1.0';
 
 
 sub FORMAT_USMARC { '1.2.840.10003.5.10' }
@@ -121,6 +122,12 @@ sub _eval_wrapper {
 	    $args->{ERR_CODE} = 100;
 	    $args->{ERR_STR} = $@->message() || $@->addinfo();
 	}
+
+	if ($@->isa('Net::Z3950::FOLIO::SurrogateDiagnostic')) {
+	    $args->{SUR_FLAG} = 1;
+	    $args->{RECORD} = ""; # To avoid an uninitialized value warning in SimpleServer.pm
+	}
+
     } elsif ($@) {
 	# Non-ZOOM exceptions may be generated by the Perl
 	# interpreter, for example if we try to call a method that
@@ -294,7 +301,7 @@ sub launch_server {
 
 
 sub _throw {
-    my($code, $addinfo, $diagset) = @_;
+    my($code, $addinfo, $diagset, $isSurrogate) = @_;
     $diagset ||= "Bib-1";
 
     # HTTP body for errors is sometimes a plain string, sometimes a JSON structure
@@ -303,7 +310,11 @@ sub _throw {
 	$addinfo = $obj->{errors} ? $obj->{errors}->[0]->{message} : $obj->{errorMessage};
     }
 
-    die new ZOOM::Exception($code, undef, $addinfo, $diagset);
+    if ($isSurrogate) {
+	die new Net::Z3950::FOLIO::SurrogateDiagnostic($code, undef, $addinfo, $diagset);
+    } else {
+	die new ZOOM::Exception($code, undef, $addinfo, $diagset);
+    }
 }
 
 
